@@ -6,14 +6,13 @@ package Devel::Dumpvar;
 # but is designed to be easier to use, more accessible, and more
 # upgradable without upgrading perl itself.
 
-use 5.005;
+use 5.006;
 use strict;
-use UNIVERSAL 'isa';
-use Scalar::Util ();
+use Scalar::Util 1.18 ();
 
 use vars qw{$VERSION};
 BEGIN {
-	$VERSION = '0.02';
+	$VERSION = '1.05';
 }
 
 
@@ -24,7 +23,7 @@ BEGIN {
 # Constructor and Accessors
 
 sub new {
-	my $class = ref $_[0] ? shift : shift;
+	my $class   = shift;
 	my %options = @_;
 
 	# Create the basic object
@@ -53,7 +52,7 @@ sub to {
 	}
 
 	# Is it something we can print to
-	if ( ref $to and UNIVERSAL::can( $to, 'print' ) ) {
+	if ( Scalar::Util::blessed($to) and $to->can('print') ) {
 		$self->{to} = $to;
 		return 1;
 	}
@@ -99,18 +98,16 @@ sub dump {
 }
 
 sub _dump_scalar {
-	my $self = shift;
-	my $value = isa( $_[0], 'SCALAR' ) ? shift
-		: die "Bad argument to _dump_scalar";
+	my $self  = shift;
+	my $value = shift;
 
 	# Print the printable form of the scalar
 	$self->_print( "$self->{indent}-> " . $self->_scalar($$value) );
 }
 
 sub _dump_ref {
-	my $self = shift;
-	my $value = isa( $_[0], 'REF' ) ? ${shift()}
-		: die "Bad argument to _dump_array";
+	my $self  = shift;
+	my $value = ${shift()};
 
 	# Print the current line
 	$self->_print( "$self->{indent}-> " . $self->_refstring($value) );
@@ -120,9 +117,8 @@ sub _dump_ref {
 }
 
 sub _dump_array {
-	my $self = shift;
-	my $array_ref = isa( $_[0], 'ARRAY' ) ? shift
-		: die "Bad argument to _dump_array";
+	my $self      = shift;
+	my $array_ref = shift;
 
 	# Handle the null array
 	unless ( @$array_ref ) {
@@ -148,13 +144,11 @@ sub _dump_array {
 }
 
 sub _dump_hash {
-	my $self = shift;
-	my $hash_ref = isa( $_[0], 'HASH' ) ? shift
-		: die "Bad argument to _dump_hash";
+	my $self     = shift;
+	my $hash_ref = shift;
 
-	foreach ( sort keys %$hash_ref ) {
-		my $key   = $self->_scalar( $_ );
-		my $value = $hash_ref->{$_};
+	foreach my $key ( sort keys %$hash_ref ) {
+		my $value = $hash_ref->{$key};
 
 		# Handle scalar values
 		unless ( ref $value ) {
@@ -172,15 +166,12 @@ sub _dump_hash {
 }
 
 sub _dump_code {
-	my $self = shift;
-	my $value = isa( $_[0], 'CODE' ) ? shift
-		: die "Bad argument to _dump_code";
-
+	my $self  = shift;
 	$self->_print( "$self->{indent}-> Sub detail listing unsupported" );
 }
 
 sub _dump_child {
-	my $self = shift;
+	my $self  = shift;
 	my $value = ref $_[0] ? shift
 		: die "Bad argument to _dump_child";
 
@@ -214,7 +205,7 @@ sub _dump_child {
 	} elsif ( $type eq 'CODE' ) {
 		$self->_dump_code( $value );
 	} else {
-		die "ARRAY -> $type not supported";
+		warn "ARRAY -> $type not supported";
 	}
 
 	# Remove indent
@@ -231,22 +222,65 @@ sub _dump_child {
 # Get the display string for a scalar value
 sub _scalar {
 	my $self = shift;
-	my $value = shift;
+	my $v    = shift;
 
 	# Shortcuts
-	return 'undef' unless defined $value;
-	return "''" unless length $value;
+	return 'undef' unless defined $v;
+	return "''"    unless length  $v;
 
 	# Is it a number?
-	if ( Scalar::Util::looks_like_number( $value ) ) {
+	if ( Scalar::Util::looks_like_number($v) ) {
 		# Show as-is
-		return $value;
+		return $v;
 	}
 
-	# Escape the scalar
-	### FINISH THIS
+	# Auto-detect the tick to use
+	my $tick = "'";
+	if ( ord('A') == 193 ) {
+		if ( $v =~ /[\000-\011]/ or $v =~ /[\013-\024\31-\037\177]/ ) {
+			$tick = '"';
+		} else {
+			$tick = "'";
+		}
+	} else {
+		if ( $v =~ /[\000-\011\013-\037\177]/ ) {
+			$tick = '"';
+		} else {
+			$tick = "'";
+		}
+	}
 
-	"'" . $value . "'";
+	# Tick-specific escaping
+	if ( $tick eq "'" ) {
+		$v =~ s/([\'\\])/\\$1/g;
+	} else {
+		$v =~ s/([\"\\\$\@])/\\$1/g;
+		$v =~ s/\033/\\e/g;
+		if ( ord('A') == 193 ) { # EBCDIC.
+			$v =~ s/([\000-\037\177])/'\\c'.chr(193)/eg; # Unfinished.
+		} else {
+			$v =~ s/([\000-\037\177])/'\\c'._scalar_ord($1)/eg;
+		}
+	}
+
+	# Unicode and high-bit escaping
+	$v = _scalar_unicode($v);
+	$v =~ s/([\200-\377])/'\\'.sprintf('%3o',ord($1))/eg;
+
+	return "${tick}${v}${tick}";
+}
+
+sub _scalar_ord {
+	my $chr = shift;
+	$chr = chr(ord($chr)^64);
+	$chr =~ s{\\}{\\\\}g;
+	return $chr;
+}
+
+sub _scalar_unicode {
+	join( "",
+	map { $_ > 255 ? sprintf("\\x{%04X}", $_) : chr($_) }
+	unpack("U*", $_[0]));
 }
 
 sub _refstring {
@@ -281,7 +315,7 @@ sub _print {
 		# Handle the "return data" case
 		$self->{return} .= $line;
 
-	} elsif ( UNIVERSAL::can( $self->{to}, 'print' ) ) {
+	} elsif ( Scalar::Util::blessed($self->{to}) and $self->{to}->can('print') ) {
 		# If we have a we something we can print to, do so
 		$self->{to}->print( $line );
 
@@ -329,18 +363,18 @@ They do this with a variety of different focuses, such as human
 readability, the ability to execute the dumped code directly, or
 to minimize the size of the dumped data.
 
-Expect for the one contained in the debugger, in the file dumpvar.pl.
-This is a readily human-readable form, highly useful for debugging,
+Excect for the one contained in the debugger, in the file dumpvar.pl.
+This is a much more human-readable form, highly useful for debugging,
 containing a lot of extra information without the burden of needing to
-be re-assembled into the original data.
+allow the dump to be re-assembled into the original data.
 
 The main downside of the dumper in the perl-debugger is that the
 dumpvar.pl script is not really a readily loadable and useable module.
 It has dedicated hooks from and to the debugger, and spans across
 multiple namespaces, including main::.
 
-Devel::Dumpvar is a pure object-orientated implementation of the
-same functionality. This makes it much more usable version to use
+Devel::Dumpvar is a pure object-orientated reimplementation of the
+same functionality. This makes it much more versatile version to use
 for dumping information to debug log files or other uses where you
 don't need to reassemble the data.
 
@@ -384,17 +418,18 @@ afterwards.
 
 Bugs should be reported via the CPAN bug tracker at
 
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Devel%3ADumpvar>
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Devel-Dumpvar>
 
-For other issues, contact the author
+For other issues, or commercial enhancement or support, contact the author.
 
 =head1 AUTHORS
 
-Adam Kennedy (Maintainer), L<http://ali.as/>, cpan@ali.as
+Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 Adam Kennedy. All rights reserved.
+Copyright 2004 - 2009 Adam Kennedy.
+
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
